@@ -41,6 +41,8 @@ struct modem_info {
 	char *syspath;
 	char *devname;
 	char *driver;
+	char *vendor;
+	char *model;
 	GSList *devices;
 	struct ofono_modem *modem;
 	const char *sysattr;
@@ -136,7 +138,7 @@ static gboolean setup_mbm(struct modem_info *modem)
 
 static gboolean setup_hso(struct modem_info *modem)
 {
-	const char *control = NULL, *application = NULL, *network = NULL;
+	const char *ctl = NULL, *app = NULL, *mdm = NULL, *net = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
@@ -148,30 +150,74 @@ static gboolean setup_hso(struct modem_info *modem)
 				info->number, info->label, info->sysattr);
 
 		if (g_strcmp0(info->sysattr, "Control") == 0)
-			control = info->devnode;
+			ctl = info->devnode;
 		else if (g_strcmp0(info->sysattr, "Application") == 0)
-			application = info->devnode;
+			app = info->devnode;
+		else if (g_strcmp0(info->sysattr, "Modem") == 0)
+			mdm = info->devnode;
 		else if (info->sysattr == NULL &&
 				g_str_has_prefix(info->devnode, "hso") == TRUE)
-			network = info->devnode;
+			net = info->devnode;
 	}
 
-	if (control == NULL || application == NULL)
+	if (ctl == NULL || app == NULL)
 		return FALSE;
 
-	DBG("control=%s application=%s network=%s",
-				control, application, network);
+	DBG("control=%s application=%s modem=%s network=%s",
+						ctl, app, mdm, net);
 
-	ofono_modem_set_string(modem->modem, "ControlPort", control);
-	ofono_modem_set_string(modem->modem, "ApplicationPort", application);
-	ofono_modem_set_string(modem->modem, "NetworkInterface", network);
+	ofono_modem_set_string(modem->modem, "Control", ctl);
+	ofono_modem_set_string(modem->modem, "Application", app);
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
 
 	return TRUE;
 }
 
 static gboolean setup_gobi(struct modem_info *modem)
 {
-	const char *device = NULL, *gps = NULL, *qcdm = NULL;
+	const char *qmi = NULL, *mdm = NULL, *net = NULL;
+	const char *gps = NULL, *diag = NULL;
+	GSList *list;
+
+	DBG("%s", modem->syspath);
+
+	for (list = modem->devices; list; list = list->next) {
+		struct device_info *info = list->data;
+
+		DBG("%s %s %s %s", info->devnode, info->interface,
+						info->number, info->label);
+
+		if (g_strcmp0(info->interface, "255/255/255") == 0) {
+			if (info->number == NULL)
+				qmi = info->devnode;
+			else if (g_strcmp0(info->number, "00") == 0)
+				net = info->devnode;
+			else if (g_strcmp0(info->number, "01") == 0)
+				diag = info->devnode;
+			else if (g_strcmp0(info->number, "02") == 0)
+				mdm = info->devnode;
+			else if (g_strcmp0(info->number, "03") == 0)
+				gps = info->devnode;
+		}
+	}
+
+	if (qmi == NULL || mdm == NULL || net == NULL)
+		return FALSE;
+
+	DBG("qmi=%s net=%s mdm=%s gps=%s diag=%s", qmi, net, mdm, gps, diag);
+
+	ofono_modem_set_string(modem->modem, "Device", qmi);
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "Diag", diag);
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
+
+	return TRUE;
+}
+
+static gboolean setup_sierra(struct modem_info *modem)
+{
+	const char *mdm = NULL, *app = NULL, *net = NULL, *diag = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
@@ -184,27 +230,32 @@ static gboolean setup_gobi(struct modem_info *modem)
 
 		if (g_strcmp0(info->interface, "255/255/255") == 0) {
 			if (g_strcmp0(info->number, "01") == 0)
-				qcdm = info->devnode;
-			else if (g_strcmp0(info->number, "02") == 0)
-				device = info->devnode;
-			else if (g_strcmp0(info->number, "03") == 0)
-				gps = info->devnode;
+				diag = info->devnode;
+			if (g_strcmp0(info->number, "03") == 0)
+				mdm = info->devnode;
+			else if (g_strcmp0(info->number, "04") == 0)
+				app = info->devnode;
+			else if (g_strcmp0(info->number, "07") == 0)
+				net = info->devnode;
 		}
 	}
 
-	if (device == NULL)
+	if (mdm == NULL || net == NULL)
 		return FALSE;
 
-	DBG("device=%s gps=%s qcdm=%s", device, gps, qcdm);
+	DBG("modem=%s app=%s net=%s diag=%s", mdm, app, net, diag);
 
-	ofono_modem_set_string(modem->modem, "Device", device);
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "App", app);
+	ofono_modem_set_string(modem->modem, "Diag", diag);
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
 
 	return TRUE;
 }
 
-static gboolean setup_sierra(struct modem_info *modem)
+static gboolean setup_option(struct modem_info *modem)
 {
-	const char *device = NULL;
+	const char *aux = NULL, *mdm = NULL, *diag = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
@@ -215,26 +266,33 @@ static gboolean setup_sierra(struct modem_info *modem)
 		DBG("%s %s %s %s", info->devnode, info->interface,
 						info->number, info->label);
 
-		if (g_strcmp0(info->interface, "255/255/255") == 0 &&
-					g_strcmp0(info->number, "03") == 0) {
-			device = info->devnode;
-			break;
+		if (g_strcmp0(info->interface, "255/255/255") == 0) {
+			if (g_strcmp0(info->number, "00") == 0)
+				mdm = info->devnode;
+			else if (g_strcmp0(info->number, "01") == 0)
+				diag = info->devnode;
+			else if (g_strcmp0(info->number, "02") == 0)
+				aux = info->devnode;
 		}
+
 	}
 
-	if (device == NULL)
+	if (aux == NULL || mdm == NULL)
 		return FALSE;
 
-	DBG("device=%s", device);
+	DBG("aux=%s modem=%s diag=%s", aux, mdm, diag);
 
-	ofono_modem_set_string(modem->modem, "Device", device);
+	ofono_modem_set_string(modem->modem, "Aux", aux);
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "Diag", diag);
 
 	return TRUE;
 }
 
 static gboolean setup_huawei(struct modem_info *modem)
 {
-	const char *mdm = NULL, *pcui = NULL, *diag = NULL;
+	const char *qmi = NULL, *mdm = NULL, *net = NULL;
+	const char *pcui = NULL, *diag = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
@@ -247,22 +305,25 @@ static gboolean setup_huawei(struct modem_info *modem)
 
 		if (g_strcmp0(info->label, "modem") == 0 ||
 				g_strcmp0(info->interface, "255/1/1") == 0 ||
-				g_strcmp0(info->interface, "255/2/1") == 0) {
+				g_strcmp0(info->interface, "255/2/1") == 0 ||
+				g_strcmp0(info->interface, "255/1/49") == 0) {
 			mdm = info->devnode;
-			if (pcui != NULL && diag != NULL)
-				break;
 		} else if (g_strcmp0(info->label, "pcui") == 0 ||
 				g_strcmp0(info->interface, "255/1/2") == 0 ||
-				g_strcmp0(info->interface, "255/2/2") == 0) {
+				g_strcmp0(info->interface, "255/2/2") == 0 ||
+				g_strcmp0(info->interface, "255/1/50") == 0) {
 			pcui = info->devnode;
-			if (mdm != NULL && diag != NULL)
-				break;
 		} else if (g_strcmp0(info->label, "diag") == 0 ||
 				g_strcmp0(info->interface, "255/1/3") == 0 ||
-				g_strcmp0(info->interface, "255/2/3") == 0) {
+				g_strcmp0(info->interface, "255/2/3") == 0 ||
+				g_strcmp0(info->interface, "255/1/51") == 0) {
 			diag = info->devnode;
-			if (mdm != NULL && pcui != NULL)
-				break;
+		} else if (g_strcmp0(info->interface, "255/1/8") == 0 ||
+				g_strcmp0(info->interface, "255/1/56") == 0) {
+			net = info->devnode;
+		} else if (g_strcmp0(info->interface, "255/1/9") == 0 ||
+				g_strcmp0(info->interface, "255/1/57") == 0) {
+			qmi = info->devnode;
 		} else if (g_strcmp0(info->interface, "255/255/255") == 0) {
 			if (g_strcmp0(info->number, "00") == 0)
 				mdm = info->devnode;
@@ -277,14 +338,22 @@ static gboolean setup_huawei(struct modem_info *modem)
 		}
 	}
 
+	if (qmi != NULL && net != NULL) {
+		ofono_modem_set_driver(modem->modem, "gobi");
+		goto done;
+	}
+
 	if (mdm == NULL || pcui == NULL)
 		return FALSE;
 
-	DBG("modem=%s pcui=%s diag=%s", mdm, pcui, diag);
+done:
+	DBG("mdm=%s pcui=%s diag=%s qmi=%s net=%s", mdm, pcui, diag, qmi, net);
 
+	ofono_modem_set_string(modem->modem, "Device", qmi);
 	ofono_modem_set_string(modem->modem, "Modem", mdm);
 	ofono_modem_set_string(modem->modem, "Pcui", pcui);
 	ofono_modem_set_string(modem->modem, "Diag", diag);
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
 
 	return TRUE;
 }
@@ -356,6 +425,50 @@ static gboolean setup_linktop(struct modem_info *modem)
 	return TRUE;
 }
 
+static gboolean setup_icera(struct modem_info *modem)
+{
+	const char *aux = NULL, *mdm = NULL, *net = NULL;
+	GSList *list;
+
+	DBG("%s", modem->syspath);
+
+	for (list = modem->devices; list; list = list->next) {
+		struct device_info *info = list->data;
+
+		DBG("%s %s %s %s", info->devnode, info->interface,
+						info->number, info->label);
+
+		if (g_strcmp0(info->interface, "2/2/1") == 0) {
+			if (g_strcmp0(info->number, "00") == 0)
+				aux = info->devnode;
+			else if (g_strcmp0(info->number, "01") == 0)
+				aux = info->devnode;
+			else if (g_strcmp0(info->number, "02") == 0)
+				mdm = info->devnode;
+			else if (g_strcmp0(info->number, "03") == 0)
+				mdm = info->devnode;
+		} else if (g_strcmp0(info->interface, "2/6/0") == 0) {
+			if (g_strcmp0(info->number, "05") == 0)
+				net = info->devnode;
+			else if (g_strcmp0(info->number, "06") == 0)
+				net = info->devnode;
+			else if (g_strcmp0(info->number, "07") == 0)
+				net = info->devnode;
+		}
+	}
+
+	if (aux == NULL || mdm == NULL)
+		return FALSE;
+
+	DBG("aux=%s modem=%s net=%s", aux, mdm, net);
+
+	ofono_modem_set_string(modem->modem, "Aux", aux);
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
+
+	return TRUE;
+}
+
 static gboolean setup_alcatel(struct modem_info *modem)
 {
 	const char *aux = NULL, *mdm = NULL;
@@ -515,19 +628,72 @@ static gboolean setup_telit(struct modem_info *modem)
 	DBG("modem=%s aux=%s gps=%s diag=%s", mdm, aux, gps, diag);
 
 	ofono_modem_set_string(modem->modem, "Modem", mdm);
-	ofono_modem_set_string(modem->modem, "Data", aux);
+	ofono_modem_set_string(modem->modem, "Aux", aux);
 	ofono_modem_set_string(modem->modem, "GPS", gps);
 
 	return TRUE;
 }
 
-static gboolean setup_zte(struct modem_info *modem)
+static gboolean setup_simcom(struct modem_info *modem)
 {
-	const char *aux = NULL, *mdm = NULL, *qcdm = NULL;
+	const char *mdm = NULL, *aux = NULL, *gps = NULL, *diag = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
 
+	for (list = modem->devices; list; list = list->next) {
+		struct device_info *info = list->data;
+
+		DBG("%s %s %s %s", info->devnode, info->interface,
+						info->number, info->label);
+
+		if (g_strcmp0(info->label, "aux") == 0) {
+			aux = info->devnode;
+			if (mdm != NULL)
+				break;
+		} else if (g_strcmp0(info->label, "modem") == 0) {
+			mdm = info->devnode;
+			if (aux != NULL)
+				break;
+		} else if (g_strcmp0(info->interface, "255/255/255") == 0) {
+			if (g_strcmp0(info->number, "00") == 0)
+				diag = info->devnode;
+			else if (g_strcmp0(info->number, "01") == 0)
+				gps = info->devnode;
+			else if (g_strcmp0(info->number, "02") == 0)
+				aux = info->devnode;
+			else if (g_strcmp0(info->number, "03") == 0)
+				mdm = info->devnode;
+		}
+	}
+
+	if (aux == NULL || mdm == NULL)
+		return FALSE;
+
+	DBG("modem=%s aux=%s gps=%s diag=%s", mdm, aux, gps, diag);
+
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "Data", aux);
+	ofono_modem_set_string(modem->modem, "GPS", gps);
+
+	return TRUE;
+}
+
+static gboolean setup_zte(struct modem_info *modem)
+{
+	const char *aux = NULL, *mdm = NULL, *qcdm = NULL;
+	const char *modem_intf;
+	GSList *list;
+
+	DBG("%s", modem->syspath);
+
+	if (g_strcmp0(modem->model, "0016") == 0 ||
+				g_strcmp0(modem->model, "0017") == 0 ||
+				g_strcmp0(modem->model, "0117") == 0)
+		modem_intf = "02";
+	else
+		modem_intf = "03";
+
 	for (list = modem->devices; list; list = list->next) {
 		struct device_info *info = list->data;
 
@@ -547,9 +713,7 @@ static gboolean setup_zte(struct modem_info *modem)
 				qcdm = info->devnode;
 			else if (g_strcmp0(info->number, "01") == 0)
 				aux = info->devnode;
-			else if (g_strcmp0(info->number, "02") == 0)
-				mdm = info->devnode;
-			else if (g_strcmp0(info->number, "03") == 0)
+			else if (g_strcmp0(info->number, modem_intf) == 0)
 				mdm = info->devnode;
 		}
 	}
@@ -603,10 +767,10 @@ static struct {
 	{ "isiusb",	setup_isi,	"type"			},
 	{ "mbm",	setup_mbm,	"device/interface"	},
 	{ "hso",	setup_hso,	"hsotype"		},
-	{ "gobi",	setup_gobi,	},
+	{ "gobi",	setup_gobi	},
 	{ "sierra",	setup_sierra	},
+	{ "option",	setup_option	},
 	{ "huawei",	setup_huawei	},
-	{ "huaweicdma",	setup_huawei	},
 	{ "speedupcdma",setup_speedup	},
 	{ "speedup",	setup_speedup	},
 	{ "linktop",	setup_linktop	},
@@ -614,7 +778,9 @@ static struct {
 	{ "novatel",	setup_novatel	},
 	{ "nokia",	setup_nokia	},
 	{ "telit",	setup_telit	},
+	{ "simcom",	setup_simcom	},
 	{ "zte",	setup_zte	},
+	{ "icera",	setup_icera	},
 	{ "samsung",	setup_samsung	},
 	{ }
 };
@@ -663,6 +829,8 @@ static void destroy_modem(gpointer data)
 	g_free(modem->syspath);
 	g_free(modem->devname);
 	g_free(modem->driver);
+	g_free(modem->vendor);
+	g_free(modem->model);
 	g_free(modem);
 }
 
@@ -705,7 +873,8 @@ static gint compare_device(gconstpointer a, gconstpointer b)
 }
 
 static void add_device(const char *syspath, const char *devname,
-			const char *driver, struct udev_device *device)
+			const char *driver, const char *vendor,
+			const char *model, struct udev_device *device)
 {
 	struct udev_device *intf;
 	const char *devpath, *devnode, *interface, *number, *label, *sysattr;
@@ -737,6 +906,8 @@ static void add_device(const char *syspath, const char *devname,
 		modem->syspath = g_strdup(syspath);
 		modem->devname = g_strdup(devname);
 		modem->driver = g_strdup(driver);
+		modem->vendor = g_strdup(vendor);
+		modem->model = g_strdup(model);
 
 		modem->sysattr = get_sysattr(driver);
 
@@ -753,6 +924,7 @@ static void add_device(const char *syspath, const char *devname,
 	else
 		sysattr = NULL;
 
+	DBG("%s", syspath);
 	DBG("%s", devpath);
 	DBG("%s (%s) %s [%s] ==> %s %s", devnode, driver,
 					interface, number, label, sysattr);
@@ -780,8 +952,14 @@ static struct {
 } vendor_list[] = {
 	{ "isiusb",	"cdc_phonet"			},
 	{ "linktop",	"cdc_acm",	"230d"		},
+	{ "icera",	"cdc_acm",	"19d2"		},
+	{ "icera",	"cdc_ether",	"19d2"		},
+	{ "icera",	"cdc_acm",	"04e8", "6872"	},
+	{ "icera",	"cdc_ether",	"04e8", "6872"	},
+	{ "icera",	"cdc_acm",	"0421", "0633"	},
+	{ "icera",	"cdc_ether",	"0421", "0633"	},
 	{ "mbm",	"cdc_acm",	"0bdb"		},
-	{ "mbm"		"cdc_ether",	"0bdb"		},
+	{ "mbm",	"cdc_ether",	"0bdb"		},
 	{ "mbm",	"cdc_acm",	"0fce"		},
 	{ "mbm",	"cdc_ether",	"0fce"		},
 	{ "mbm",	"cdc_acm",	"413c"		},
@@ -791,19 +969,23 @@ static struct {
 	{ "mbm",	"cdc_acm",	"0930"		},
 	{ "mbm",	"cdc_ether",	"0930"		},
 	{ "hso",	"hso"				},
+	{ "gobi",	"qmi_wwan"			},
 	{ "gobi",	"qcserial"			},
 	{ "sierra",	"sierra"			},
-	{ "huawei",	"option",	"201e", "2009"	},
+	{ "sierra",	"sierra_net"			},
+	{ "option",	"option",	"0af0"		},
+	{ "huawei",	"option",	"201e"		},
+	{ "huawei",	"cdc_wdm",	"12d1"		},
 	{ "huawei",	"cdc_ether",	"12d1"		},
+	{ "huawei",	"qmi_wwan",	"12d1"		},
 	{ "huawei",	"option",	"12d1"		},
-	{ "huaweicdma",	"option",	"12d1", "140b"	},
-	{ "huaweicdma", "option",	"201e"		},
 	{ "speedupcdma","option",	"1c9e", "9e00"	},
 	{ "speedup",	"option",	"1c9e"		},
 	{ "speedup",	"option",	"2020"		},
 	{ "alcatel",	"option",	"1bbb", "0017"	},
 	{ "novatel",	"option",	"1410"		},
 	{ "zte",	"option",	"19d2"		},
+	{ "simcom",	"option",	"05c6", "9000"	},
 	{ "telit",	"usbserial",	"1bc7"		},
 	{ "telit",	"option",	"1bc7"		},
 	{ "nokia",	"option",	"0421", "060e"	},
@@ -816,7 +998,8 @@ static struct {
 static void check_usb_device(struct udev_device *device)
 {
 	struct udev_device *usb_device;
-	const char *driver, *syspath, *devname;
+	const char *syspath, *devname, *driver;
+	const char *vendor = NULL, *model = NULL;
 
 	usb_device = udev_device_get_parent_with_subsystem_devtype(device,
 							"usb", "usb_device");
@@ -837,16 +1020,23 @@ static void check_usb_device(struct udev_device *device)
 		unsigned int i;
 
 		drv = udev_device_get_property_value(device, "ID_USB_DRIVER");
-		if (drv == NULL)
-			return;
+		if (drv == NULL) {
+			drv = udev_device_get_driver(device);
+			if (drv == NULL) {
+				struct udev_device *parent;
+
+				parent = udev_device_get_parent(device);
+				if (parent == NULL)
+					return;
+
+				drv = udev_device_get_driver(parent);
+				if (drv == NULL)
+					return;
+			}
+		}
 
 		vid = udev_device_get_property_value(device, "ID_VENDOR_ID");
-		if (vid == NULL)
-			return;
-
 		pid = udev_device_get_property_value(device, "ID_MODEL_ID");
-		if (pid == NULL)
-			return;
 
 		DBG("%s [%s:%s]", drv, vid, pid);
 
@@ -856,18 +1046,26 @@ static void check_usb_device(struct udev_device *device)
 
 			if (vendor_list[i].vid == NULL) {
 				driver = vendor_list[i].driver;
-				break;
+				vendor = vid;
+				model = pid;
+				continue;
 			}
 
+			if (vid == NULL || pid == NULL)
+				continue;
+
 			if (g_str_equal(vendor_list[i].vid, vid) == TRUE) {
 				if (vendor_list[i].pid == NULL) {
-					if (driver == NULL)
-						driver = vendor_list[i].driver;
+					driver = vendor_list[i].driver;
+					vendor = vid;
+					model = pid;
 					continue;
 				}
 
 				if (g_strcmp0(vendor_list[i].pid, pid) == 0) {
 					driver = vendor_list[i].driver;
+					vendor = vid;
+					model = pid;
 					break;
 				}
 			}
@@ -877,7 +1075,7 @@ static void check_usb_device(struct udev_device *device)
 			return;
 	}
 
-	add_device(syspath, devname, driver, device);
+	add_device(syspath, devname, driver, vendor, model, device);
 }
 
 static void check_device(struct udev_device *device)
@@ -885,8 +1083,11 @@ static void check_device(struct udev_device *device)
 	const char *bus;
 
 	bus = udev_device_get_property_value(device, "ID_BUS");
-	if (bus == NULL)
-		return;
+	if (bus == NULL) {
+		bus = udev_device_get_subsystem(device);
+		if (bus == NULL)
+			return;
+	}
 
 	if (g_str_equal(bus, "usb") == TRUE)
 		check_usb_device(device);
@@ -937,6 +1138,7 @@ static void enumerate_devices(struct udev *context)
 		return;
 
 	udev_enumerate_add_match_subsystem(enumerate, "tty");
+	udev_enumerate_add_match_subsystem(enumerate, "usb");
 	udev_enumerate_add_match_subsystem(enumerate, "net");
 
 	udev_enumerate_scan_devices(enumerate);
@@ -1058,6 +1260,7 @@ static int detect_init(void)
 						NULL, destroy_modem);
 
 	udev_monitor_filter_add_match_subsystem_devtype(udev_mon, "tty", NULL);
+	udev_monitor_filter_add_match_subsystem_devtype(udev_mon, "usb", NULL);
 	udev_monitor_filter_add_match_subsystem_devtype(udev_mon, "net", NULL);
 
 	udev_monitor_filter_update(udev_mon);
